SCREEN_WIDTH = 1900
SCREEN_HEIGHT = 1000

RECT_WIDTH = 25
RECT_HEIGHT = 25

GAP = 5

APPLES = 15
SNAKES = 10
TIMER_RATIO = 0.4

GRID_COLOR = (10, 10, 10)
BACKGROUND_COLOR = (100, 200, 255)
SNAKES_COLOR = (255, 100, 0)

BLACK = (0, 0, 0)
RED = (255, 0, 0)
GREEN = (0, 255, 0)
BLUE = (0, 0, 255)
WHITE = (255, 255, 255)
LIGHT_GREEN = (136, 255, 127)
SKY_BLUE = (127, 218, 255)
YELLOW = (242, 255, 127)
PINK = (255, 127, 240)
BROWN = (132, 39, 0)
DARK_GREEN = (28, 56, 24)

COLORS = (GREEN, BLUE, LIGHT_GREEN, YELLOW, PINK, BROWN, DARK_GREEN)

MAP_HEIGHT = SCREEN_HEIGHT // RECT_HEIGHT
MAP_WIDTH = SCREEN_WIDTH // RECT_WIDTH
